package com.citra.pivot.demo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DateFormat;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;

import com.citra.pivot.DefaultPivotTableModel;
import com.citra.pivot.PivotDataModel;
import com.citra.pivot.PivotTable;
import com.citra.pivot.olap.DefaultOlapProvider;
import com.citra.pivot.olap.OlapDataModel;
import com.citra.pivot.olap.OlapPivotTable;
import com.citra.pivot.olap.OlapProvider;
import com.citra.pivot.olap.data.DataSource;
import com.citra.pivot.olap.gui.HeaderDndSupport;
import com.citra.pivot.olap.gui.OlapAssistant;
import com.citra.pivot.olap.gui.OlapCubePanel;
import com.citra.pivot.olap.gui.OlapFilterPanel;
import com.citra.pivot.olap.metadata.OlapCube;
import com.citra.pivot.olap.remote.RemoteOlapModel;
import com.citra.pivot.olap.remote.RemoteOlapPanel;
import com.citra.pivot.olap.remote.RemoteOlapStyle;
import com.citra.pivot.version.PivotVersion;
import com.citra.table.AdvancedJScrollPane;
import com.sun.java.swing.plaf.windows.WindowsLookAndFeel;
/**
 *
 */
public abstract class AbstractOlapDemo {
	/** the underlying data source */
	protected DataSource dataSource;
	/** the pivot table */
	protected PivotTable pivotTable;
	/** whether to use a remote model */
	protected boolean useRemote;
	/** the remote model */
	protected RemoteOlapModel remoteModel;
/**
 *
 */
public AbstractOlapDemo() {
	this(false);
}
/**
 *
 */
public AbstractOlapDemo(boolean useRemote) {
	this.useRemote = useRemote;
}
/**
 *
 */
protected abstract DataSource createDataSource();
/**
 *
 */
private PivotTable createTable(OlapProvider provider) {
	//create pivot table
	PivotTable pivotTable;
	if (useRemote) {
		remoteModel = new RemoteOlapModel(new OlapDataModel(provider));
		pivotTable = new OlapPivotTable(remoteModel);
	}
	else {
		//create pivot table
		pivotTable = new OlapPivotTable(provider);
	}

	//set a color for the odd rows of the table - alternating
	Color c = new Color(194, 194, 194);
	pivotTable.setOddColor(c);

	return pivotTable;
}
/**
 *
 */
protected void customizeTable(PivotTable pivotTable, DefaultOlapProvider provider) {
}
/**
 *
 */
public DataSource getDataSource() {
	if (dataSource == null) {
		dataSource = createDataSource();
	}
	return dataSource;
}
/**
 *
 */
protected Container getFramePanel(Container tablePanel) {
	return tablePanel;
}
/**
 *
 */
protected void setUseRemote(boolean useRemote) {
	if (useRemote) {
		DefaultPivotTableModel pivotModel = (DefaultPivotTableModel) pivotTable.getPivotTableModel();
		PivotDataModel dataModel = pivotModel.getDataModel();
		if (dataModel instanceof OlapDataModel) {
			if (remoteModel == null) remoteModel = new RemoteOlapModel((OlapDataModel) dataModel);
			else remoteModel.setOlapDataModel((OlapDataModel) dataModel);
			pivotModel.setDataModel(remoteModel);
		}
	}
	else {
		DefaultPivotTableModel pivotModel = (DefaultPivotTableModel) pivotTable.getPivotTableModel();
		PivotDataModel dataModel = pivotModel.getDataModel();
		if (dataModel == remoteModel) {
			OlapDataModel innerModel = remoteModel.getOlapDataModel();
			//release data model from RemoteOlapModel
			remoteModel.setOlapDataModel(null);
			//assign previous data model
			pivotModel.setDataModel(innerModel);
		}
	}
}
/**
 *
 */
public void show() throws Exception {

	//set the windows ui
	try {
		UIManager.setLookAndFeel(new WindowsLookAndFeel());
	} catch (Exception e) {
		System.out.println(e);
	}

	//get the data source
	dataSource = getDataSource();

	//get the first cube of the schema
	OlapCube cube = dataSource.getSchema().getCubes().getCube(0);

	//create olap provider
	DefaultOlapProvider provider = new DefaultOlapProvider(dataSource, cube);

	//create the pivot table
	pivotTable = createTable(provider);

	//customization used by AbstractOlapDemo subclasses
	customizeTable(pivotTable, provider);

	//set a new style model
	final OlapDemoStyleModel demoStyle = new OlapDemoStyleModel();
	pivotTable.setPivotStyleModel(demoStyle);

	//show columns and row grand totals
	provider.setShowColumnGrandTotal(true);
	provider.setShowRowGrandTotal(true);

	//show the row header
	pivotTable.setShowRowHeader(true);

	//always show the row and column grand totals
	pivotTable.setFixedGrandTotals(true);

	//add olap assistant's functionality
	OlapAssistant assist = new OlapAssistant(pivotTable, provider);

	//create scroll pane for table
	JScrollPane tableScroller = new AdvancedJScrollPane(pivotTable);

	//create panel for table
	JPanel tablePanel = new JPanel(new BorderLayout());
	tablePanel.add(tableScroller);
	tablePanel.setBorder(new TitledBorder("Pivot Table"));

	//create filter panel
	JPanel filterPanel = new OlapFilterPanel(provider);
	filterPanel.setBorder(new TitledBorder("Filters"));
	//create cube browser panel
	JPanel browserPanel = new OlapCubePanel(provider);
	browserPanel.setBorder(new TitledBorder(cube.getName()));
	//create a split pane to hold the cube browser and filter panel
	JSplitPane leftSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, filterPanel, browserPanel);

	//create options panel
	JPanel optionsPanel = new JPanel();
	optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.Y_AXIS));
	optionsPanel.setBorder(new TitledBorder("Options"));
	//add a checkbox for applying the table's style
	JCheckBox showStyle = new JCheckBox("show style", true);
	showStyle.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			demoStyle.setApplyStyle(!demoStyle.getApplyStyle());
			pivotTable.repaint();
			pivotTable.getTableHeader().repaint();
		}
	});
	optionsPanel.add(showStyle);
	//add a checkbox for locking grand totals
	JCheckBox fixGrandTotals = new JCheckBox("fix grand totals", true);
	fixGrandTotals.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			pivotTable.setFixedGrandTotals(!pivotTable.getFixedGrandTotals());
		}
	});
	optionsPanel.add(fixGrandTotals);
	//add a checkbox for controlling whether a remote model will be used
	final JCheckBox remote = new JCheckBox("non-freezing", useRemote);
	remote.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			setUseRemote(remote.isSelected());
		}
	});
	optionsPanel.add(remote);

	//create a panel to hold the filter, cube and options panels
	JPanel rightPanel = new JPanel(new BorderLayout());
	rightPanel.add(leftSplitPane);
	rightPanel.add(optionsPanel, BorderLayout.NORTH);

	//create a split pane for the pivot table and right panel
	JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, tablePanel, rightPanel);
	splitPane.setMinimumSize(new Dimension(600, 300));

	//apply dnd support to header
	HeaderDndSupport hdnd = new HeaderDndSupport(pivotTable.getPivotTableHeader(), provider);

	//create frame
	JFrame frame = new JFrame();
	//get the panel to add to the frame
	Container framePanel = getFramePanel(splitPane);

	//add a status panel and a remote style if a remote model is used
	if (useRemote && remoteModel != null) {
		//add status panel
		RemoteOlapPanel status = new RemoteOlapPanel();
		frame.getContentPane().add(status, BorderLayout.SOUTH);
		//add a listener to the status panel
		remoteModel.addRemoteListener(status);
	}
	//add a style that is enabled when data values are being fetched
	RemoteOlapStyle rs = new RemoteOlapStyle();
	pivotTable.getStyleModel().addStyle(rs);

	//add it to the frame's content pane
	frame.getContentPane().add(framePanel);

	String dstr = DateFormat.getDateInstance().format(PivotVersion.getVersionDate());
	frame.setTitle("Citra Pivot Demo version " + PivotVersion.getVersion() + " - " + dstr);
	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	});

	//center and resize frame
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	int width = (int) (screenSize.width * 0.75);
	int height = (int) (screenSize.height * 0.75);
	if (width <= 0) width = 700;
	if (height <= 0) height = 500;
	frame.setSize(width, height);
	Dimension d = frame.getSize();
	frame.setLocation((screenSize.width - d.width) / 2,(screenSize.height - d.height) / 2);

	//set visible
	frame.setVisible(true);

	//set splitter's divider location
	splitPane.setDividerLocation(0.8);
}
}
